// SemiTwist Compiler
// Written in the D programming language.

/** 
Author:
$(WEB www.semitwist.com, Nick Sabalausky)
*/

module stc.grammar;

import mtext;
import semitwist.util;

const char[] nullIndicator = "[null]";

interface GrammarNode
{
	public mstring getName();
}

class GrammarTerminal : GrammarNode
{
	mixin(getter("mstring", "name", "getName"));
	mixin(getter("mstring", "pattern", "getPattern"));
	
	public this(char[] name, char[] pattern)
	{
		this(mstring(name), mstring(pattern));
	}

	public this(mstring name, mstring pattern)
	{
		mixin(initMember!(name, pattern));
	}

	char[] toString()
	{
		return name.toUTF8();
	}
}

class GrammarNonTerminal : GrammarNode
{
	mixin(getter("mstring", "name", "getName"));
	mixin(getter("GrammarClause[]", "clauses", "getClauses"));
	
	public this(char[] name, GrammarClause[] clauses)
	{
		this(mstring(name), clauses);
	}
	
	public this(mstring name, GrammarClause[] clauses)
	{
		mixin(initMember!(name, clauses));
	}

	char[] toString()
	{
		char[] ret;
		char[] delim = "\n  | ";
		foreach(GrammarClause clause; clauses)
			ret ~= delim ~ clause.toString();
		if(ret.length == 0)
			ret = nullIndicator;
		return name.toUTF8() ~ " := " ~ ret;
	}
}

class GrammarClause
{
	mixin(getter("GrammarNode[]", "elements", "getElements"));
	
	public this(GrammarNode[] elements)
	{
		mixin(initMember!(elements));
	}
	
	char[] toString()
	{
		char[] ret;
		char[] delim = ",";
		foreach(GrammarNode element; elements)
			ret ~= element.getName().toUTF8 ~ delim;
		//TODO: Put this in a function
		if(ret.length >= delim.length)
		{
			if(ret[$-delim.length..$] == delim)
				ret = ret[0..$-delim.length];
		}
		if(ret.length == 0)
			ret = nullIndicator;
		return ret;
	}
}
