// SemiTwist Compiler
// Written in the D programming language.

/** 
Author:
$(WEB www.semitwist.com, Nick Sabalausky)
*/

module stc.lang.stca;

import tango.io.Stdout;
import tango.text.Regex;
import tango.text.Util;

import semitwist.util;

import stc.util;
import stc.fsm;
//import stc.lexer;
//import stc.parser;
import stc.language;
/+
class STCA : Language
{
	mixin(makeTokenTypes([
		"Equals",        // ":=="
		"Or",            // "|"
		"EOLComment",    // "//[^\n]*"
		"BlockComment",  // "/\*[^\n]*\*/"
		"Identifier",    // "[_\u\l][_\d\u\l]*"
		"Whitespace"     // "[\s]+"
		]));

	TokenType[] getTokenTypes()
	{
		TokenType[] tokTypes;
		
		tokTypes ~= new TokenType(r":==",
						delegate Token(schar[] data)
						{ return new TokEquals(data); });
		
		tokTypes ~= new TokenType(r"\|",
						delegate Token(schar[] data)
						{ return new TokOr(data); });
		
		tokTypes ~= new TokenType(r"//[^\r\n]*",
						delegate Token(schar[] data)
						{ return new TokEOLComment(data); });
		
		tokTypes ~= new TokenType(r"/\*(?\*(?>[^/])|[^\*])*\*/",
						delegate Token(schar[] data)
						{ return new TokBlockComment(data); });
		
		tokTypes ~= new TokenType(r"[_a-zA-Z][_a-zA-Z0-9]*",
						delegate Token(schar[] data)
						{ return new TokIdentifier(data); });
		
		tokTypes ~= new TokenType(r"\s+",
						delegate Token(schar[] data)
						{ return new TokWhitespace(data); });
		
		return tokTypes;
	}
}

class STCA_Old2 : Language
{
	mixin(makeTokenTypes([
		"Equals",        // ":=="
		"Or",            // "|"
		"EOLComment",    // "//[^\n]*"
		"BlockComment",  // "/\*[^\n]*\*/"
		"Identifier",    // "[_\u\l][_\d\u\l]*"
		"Whitespace"     // "[\s]+"
		]));

	TokenType[] getTokenTypes()
	{
		TokenType[] tokTypes;
		RegexOp!(schar)[][] opSets;
		RegexOp!(schar)[] opSet;
		
Stdout.formatln("--- Equals");
		opSet.length = 0;
		opSet ~= new RegexOp!(schar)(RegexOpMultiplicity.One, RegexOpInclusivity.Include, ":");
		opSet ~= new RegexOp!(schar)(RegexOpMultiplicity.One, RegexOpInclusivity.Include, "=");
		opSet ~= new RegexOp!(schar)(RegexOpMultiplicity.One, RegexOpInclusivity.Include, "=");
		opSets ~= opSet;
		
		tokTypes ~= new TokenType(
						new FSM!(schar)(opSet),
						delegate Token(schar[] data)
						{ return new TokEquals(data); });

Stdout.formatln("blah: {} (line {})", __FILE__, __LINE__);
Stdout.formatln("--- Or");
Stdout.formatln("blah: {} (line {})", __FILE__, __LINE__);
		opSet.length = 0;
		opSet ~= new RegexOp!(schar)(RegexOpMultiplicity.One, RegexOpInclusivity.Include, "|");
		opSets ~= opSet;
Stdout.formatln("blah: {} (line {})", __FILE__, __LINE__);
		
		tokTypes ~= new TokenType(
						new FSM!(schar)(opSet),
						delegate Token(schar[] data)
						{ return new TokOr(data); });

Stdout.formatln("--- EOLComment");
		opSet.length = 0;
		opSet ~= new RegexOp!(schar)(RegexOpMultiplicity.One, RegexOpInclusivity.Include, "/");
		opSet ~= new RegexOp!(schar)(RegexOpMultiplicity.One, RegexOpInclusivity.Include, "/");
		opSet ~= new RegexOp!(schar)(RegexOpMultiplicity.Any, RegexOpInclusivity.Exclude, "\n");
		opSets ~= opSet;
		
		tokTypes ~= new TokenType(
						new FSM!(schar)(opSet),
						delegate Token(schar[] data)
						{ return new TokEOLComment(data); });

Stdout.formatln("--- BlockComment");
		opSet.length = 0;
		opSet ~= new RegexOp!(schar)(RegexOpMultiplicity.One, RegexOpInclusivity.Include, "/");
		opSet ~= new RegexOp!(schar)(RegexOpMultiplicity.One, RegexOpInclusivity.Include, "*");
//		opSet ~= new RegexOp!(schar)(RegexOpMultiplicity.Any, RegexOpInclusivity.Exclude, "\n");
		opSet ~= new RegexOp!(schar)(RegexOpMultiplicity.Any, RegexOpInclusivity.Exclude, "");
		opSet ~= new RegexOp!(schar)(RegexOpMultiplicity.One, RegexOpInclusivity.Include, "*");
		opSet ~= new RegexOp!(schar)(RegexOpMultiplicity.One, RegexOpInclusivity.Include, "/");
		opSets ~= opSet;
		
		tokTypes ~= new TokenType(
						new FSM!(schar)(opSet),
						delegate Token(schar[] data)
						{ return new TokBlockComment(data); });

Stdout.formatln("--- Identifier");
		opSet.length = 0;
		opSet ~= new RegexOp!(schar)(RegexOpMultiplicity.One, RegexOpInclusivity.Include, unescapeSemiTwist!(char)(r"_\u\l"));
		opSet ~= new RegexOp!(schar)(RegexOpMultiplicity.Any, RegexOpInclusivity.Include, unescapeSemiTwist!(char)(r"_\u\l\d"));
		opSets ~= opSet;
		
		tokTypes ~= new TokenType(
						new FSM!(schar)(opSet),
						delegate Token(schar[] data)
						{ return new TokIdentifier(data); });

Stdout.formatln("--- Whitespace");
		opSet.length = 0;
		opSet ~= new RegexOp!(schar)(RegexOpMultiplicity.One, RegexOpInclusivity.Include, unescapeSemiTwist!(char)(r"\s"));
		opSet ~= new RegexOp!(schar)(RegexOpMultiplicity.Any, RegexOpInclusivity.Include, unescapeSemiTwist!(char)(r"\s"));
		opSets ~= opSet;

		tokTypes ~= new TokenType(
						new FSM!(schar)(opSet),
						delegate Token(schar[] data)
						{ return new TokWhitespace(data); });
		
		return tokTypes;
	}
}

class STCA_Old1 : Language
{
	mixin(makeTokenTypes_old([
		"Equals",
		"Or",
		"EOLComment",
		"BlockComment",
		"Identifier",
		"Whitespace"
		]));

/*
	Stdout.formatln("{,-20}", "Found \":=\" Operator");
	Stdout.formatln("{,-20}", "Found \"|\" Operator");
	Stdout.formatln("{,-20} {}", "Found EOL Comment: ", content);
	Stdout.formatln("{,-20} {}", "Found Block Comment: ", content);
	Stdout.formatln("{,-20} {}", "Found Identifier: ", content);
	Stdout.formatln("{,-20}", "Found Whitespace");
	Stdout.formatln("{,-20} {}", "Found Unknown Token: ", content);
*/

	private FSMState!(schar) genEqualsFSM()
	{
		mixin(genFSMInit("4"));

		fsm[0].addNextStateIf(    r":", fsm[1]);
		
		fsm[1].addNextStateIf(    r"=", fsm[2]);

		fsm[2].addNextStateIf(    r"=", fsm[3]);
		
		fsm[3].addNextStateIf(    r"=", FSMState!(schar).endState);
		fsm[3].addNextStateIfNot( r"=", FSMState!(schar).endState);

		return fsm[0];
	}

	private FSMState!(schar) genOrFSM()
	{
		mixin(genFSMInit("2"));

		fsm[0].addNextStateIf(    r"|", fsm[1]);
		
		fsm[1].addNextStateIf(    r"|", FSMState!(schar).endState);
		fsm[1].addNextStateIfNot( r"|", FSMState!(schar).endState);

		return fsm[0];
	}

	private FSMState!(schar) genEOLCommentFSM()
	{
		mixin(genFSMInit("3"));

		fsm[0].addNextStateIf(    r"/", fsm[1]);
		
		fsm[1].addNextStateIf(    r"/", fsm[2]);
		
		fsm[2].addNextStateIfNot( "\n", fsm[2]);
		fsm[2].addNextStateIf(    "\n", FSMState!(schar).endState);
		
		return fsm[0];
	}

	private FSMState!(schar) genBlockCommentFSM()
	{
		mixin(genFSMInit("5"));

		fsm[0].addNextStateIf(    r"/", fsm[1]);
		
		fsm[1].addNextStateIf(    r"*", fsm[2]);
		
		fsm[2].addNextStateIfNot( r"*", fsm[2]);
		fsm[2].addNextStateIf(    r"*", fsm[3]);

		fsm[3].addNextStateIf(    r"/", fsm[4]);
		fsm[3].addNextStateIfNot( r"/", fsm[2]);
		
		fsm[4].addNextStateIf(    r"/", FSMState!(schar).endState);
		fsm[4].addNextStateIfNot( r"/", FSMState!(schar).endState);
		
		return fsm[0];
	}

	private FSMState!(schar) genIdentifierFSM()
	{
		mixin(genFSMInit("2"));

		fsm[0].addNextStateIf(    unescapeSemiTwist!(char)(r"_\l\u"),   fsm[1]);
		
		fsm[1].addNextStateIf(    unescapeSemiTwist!(char)(r"_\l\u\d"), fsm[1]);
		fsm[1].addNextStateIfNot( unescapeSemiTwist!(char)(r"_\l\u\d"), FSMState!(schar).endState);
		
		return fsm[0];
	}

	private FSMState!(schar) genWhitespaceFSM()
	{
		mixin(genFSMInit("2"));

		fsm[0].addNextStateIf(    unescapeSemiTwist!(char)(r"\s"), fsm[1]);
		
		fsm[1].addNextStateIf(    unescapeSemiTwist!(char)(r"\s"), fsm[1]);
		fsm[1].addNextStateIfNot( unescapeSemiTwist!(char)(r"\s"), FSMState!(schar).endState);
		
		return fsm[0];
	}
}
+/
