// SemiTwist Compiler
// Written in the D programming language.

/** 
Author:
$(WEB www.semitwist.com, Nick Sabalausky)
*/

module stc.language;

import tango.io.Stdout;
import tango.text.Regex;
import tango.text.Util;

import semitwist.util;

import stc.util;
import stc.fsm;

// Should be in stc.language
abstract class Language
{
	abstract TokenType[] getTokenTypes();
}

abstract class Token
{
	protected schar[] content;
	//schar[] match; //TODO: Implement this

	this(schar[] content)
	{
		mixin(initMember!(content));
	}
	
	// TODO: Change to toString
	char[] getContent()
	{
		return content.dup;
	}
}

/*
// I think this method is only useful if I want to use alias args
// or varadic args.
template defineToken(char[] name, char[] content)
{
	class TokIdentifier : Token
	{
		this(schar[] content)
		{
			
		}
	}
}
*/

class TokenType
{
	private FSM!(schar) fsm;
	private schar[] regexStr;
	private Token delegate(schar[]) tokenCreator;
	
	this(FSM!(schar) fsm, Token delegate(schar[]) tokenCreator)
	{
		mixin(initMember!(fsm, tokenCreator));
	}
	
	this(schar[] regexStr, Token delegate(schar[]) tokenCreator)
	{
		mixin(initMember!(regexStr, tokenCreator));
	}
	
	FSM!(schar) getFSM()
	{
		return fsm is null ? null : fsm.dup();
	}
	
	RegExpT!(schar) getRegex()
	{
		return new RegExpT!(schar)(regexStr);
	}
	
	Token newToken(schar[] content)
	{
		return tokenCreator(content);
	}
}

/**
Intended to be used inside the definition
of a class derived from Language
*/
char[] defineToken(char[] name)
{
	char[] ctor;
	ctor =  `this(schar[] content)`~
			`{ super(content); }`;
				
	return  "static TokenType tokType"~name~";\n"
			"static class Tok"~name~" : Token\n"
			"{ "~ctor~" }\n";
}

char[] makeTokenTypes(char[][] tokenNames)
{
	char[] ret;

	foreach(size_t i, char[] name; tokenNames)
		ret ~= defineToken(name);
	
	return ret;
}

char[] makeTokenTypes_old(char[][] tokenNames)
{
	char[] ret;
	ret ~=
	"TokenType[] getTokenTypes()\n"
	"{\n"
	"	TokenType[] tokTypes;\n";

	foreach(size_t i, char[] name; tokenNames)
		ret ~= initTokenType(name);
		
	ret ~=
	"	return tokTypes;\n"
	"}\n";
	
	foreach(size_t i, char[] name; tokenNames)
		ret ~= defineToken(name);

	return ret;
}

char[] initTokenType(char[] tokenName)
{
		return
		"tokTypes ~=\n"
		"(\n"
		"	tokType"~tokenName~" =\n"
		"		new TokenType(\n"
		"			new FSM!(schar)(gen"~tokenName~"FSM()),\n"
		"			delegate Token(schar[] data)\n"
		"				{ return new Tok"~tokenName~"(data); }\n"
		"		)\n"
		");\n";
}

char[] genFSMInit(char[] numStates)
{
	return
	"size_t numStates="~numStates~";\n"
	"FSMState!(schar)[] fsm;\n"
	"for(size_t i=0; i < numStates; i++)\n"
	"	fsm ~= new LexFSMState();\n";
}
